/* == This file is part of Tomahawk Player - <http://tomahawk-player.org> ===
 *
 *   Copyright 2012, Enno Gottschalk <mrmaffen@googlemail.com>
 *
 *   Tomahawk is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Tomahawk is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Tomahawk. If not, see <http://www.gnu.org/licenses/>.
 */
package org.tomahawk.tomahawk_android.fragments;

import org.tomahawk.libtomahawk.collection.Album;
import org.tomahawk.libtomahawk.collection.Collection;
import org.tomahawk.libtomahawk.collection.Track;
import org.tomahawk.libtomahawk.collection.UserCollection;
import org.tomahawk.libtomahawk.collection.UserPlaylist;
import org.tomahawk.libtomahawk.hatchet.InfoSystem;
import org.tomahawk.libtomahawk.resolver.PipeLine;
import org.tomahawk.libtomahawk.resolver.Query;
import org.tomahawk.libtomahawk.utils.TomahawkUtils;
import org.tomahawk.tomahawk_android.activities.PlaybackActivity;
import org.tomahawk.tomahawk_android.adapters.TomahawkBaseAdapter;
import org.tomahawk.tomahawk_android.adapters.TomahawkListAdapter;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.support.v4.content.Loader;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;

import java.util.ArrayList;
import java.util.List;

/**
 * {@link TomahawkFragment} which shows a set of {@link Track}s inside its {@link
 * org.tomahawk.tomahawk_android.views.TomahawkStickyListHeadersListView}
 */
public class TracksFragment extends TomahawkFragment implements OnItemClickListener {

    boolean mShouldShowLoadingAnimation = false;

    private TrackFragmentReceiver mTrackFragmentReceiver;

    /**
     * Handles incoming broadcasts.
     */
    private class TrackFragmentReceiver extends BroadcastReceiver {

        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getAction().equals(PipeLine.PIPELINE_RESULTSREPORTED_NON_FULLTEXTQUERY)) {
                String queryId = intent.getStringExtra(PipeLine.PIPELINE_RESULTSREPORTED_QID);
                if (mCorrespondingQueryIds.containsKey(queryId)) {
                    ArrayList<Track> tracks = mPipeline.getQuery(queryId).getTrackResults();
                    if (tracks != null && tracks.size() > 0) {
                        Track track = mCorrespondingQueryIds.get(queryId);
                        if (track.getScore() < tracks.get(0).getScore()) {
                            Query.trackResultToTrack(tracks.get(0), track);
                            updateAdapter();
                        }
                    }
                }
            } else if (intent.getAction().equals(InfoSystem.INFOSYSTEM_RESULTSREPORTED)) {
                String requestId = intent
                        .getStringExtra(InfoSystem.INFOSYSTEM_RESULTSREPORTED_REQUESTID);
                if (mCurrentRequestIds.contains(requestId)) {
                }
            }
        }
    }

    /**
     * Pulls all the necessary information from the {@link Bundle}s that are being sent, when this
     * {@link TracksFragment} is created. We can access the information through getArguments().
     *
     * @param savedInstanceState If the activity is being re-initialized after previously being shut
     *                           down then this {@link Bundle} contains the data it most recently
     *                           supplied in onSaveInstanceState({@link Bundle}). Note: Otherwise it
     *                           is null.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (mTomahawkMainActivity.getCollection() != null && getArguments() != null) {
            if (getArguments().containsKey(TOMAHAWK_ALBUM_ID)
                    && getArguments().getLong(TOMAHAWK_ALBUM_ID) >= 0) {
                mAlbum = mTomahawkMainActivity.getCollection()
                        .getAlbumById(getArguments().getLong(TOMAHAWK_ALBUM_ID));
            } else if (getArguments().containsKey(TOMAHAWK_PLAYLIST_ID)
                    && getArguments().getLong(TOMAHAWK_PLAYLIST_ID) >= 0) {
                mUserPlaylist = mTomahawkMainActivity.getCollection()
                        .getCustomPlaylistById(getArguments().getLong(TOMAHAWK_PLAYLIST_ID));
            } else if (getArguments().containsKey(UserCollection.USERCOLLECTION_ALBUMCACHED)) {
                // A cached album has been given. So we try to resolve it.
                mAlbum = mTomahawkMainActivity.getCollection().getCachedAlbum();
                resolveAlbum(mAlbum);
                mShouldShowLoadingAnimation = true;
            }
        }
    }

    /**
     * Initialize and register {@link TrackFragmentReceiver}
     */
    @Override
    public void onResume() {
        super.onResume();

        if (mTrackFragmentReceiver == null) {
            mTrackFragmentReceiver = new TrackFragmentReceiver();
            IntentFilter intentFilter = new IntentFilter(Collection.COLLECTION_UPDATED);
            getActivity().registerReceiver(mTrackFragmentReceiver, intentFilter);
            intentFilter = new IntentFilter(InfoSystem.INFOSYSTEM_RESULTSREPORTED);
            getActivity().registerReceiver(mTrackFragmentReceiver, intentFilter);
            intentFilter = new IntentFilter(PipeLine.PIPELINE_RESULTSREPORTED_NON_FULLTEXTQUERY);
            getActivity().registerReceiver(mTrackFragmentReceiver, intentFilter);
        }
        if (mShouldShowLoadingAnimation) {
            startLoadingAnimation();
        }
    }

    /**
     * Unregister {@link TrackFragmentReceiver} and delete reference
     */
    @Override
    public void onPause() {
        super.onPause();

        if (mTrackFragmentReceiver != null) {
            getActivity().unregisterReceiver(mTrackFragmentReceiver);
            mTrackFragmentReceiver = null;
        }
    }

    /**
     * Called every time an item inside the {@link org.tomahawk.tomahawk_android.views.TomahawkStickyListHeadersListView}
     * is clicked
     *
     * @param parent   The AdapterView where the click happened.
     * @param view     The view within the AdapterView that was clicked (this will be a view
     *                 provided by the adapter)
     * @param position The position of the view in the adapter.
     * @param id       The row id of the item that was clicked.
     */
    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        position -= getListView().getHeaderViewsCount();
        if (position >= 0) {
            if (getListAdapter().getItem(position) instanceof Track && ((Track) getListAdapter()
                    .getItem(position)).isResolved()) {
                ArrayList<Track> tracks = new ArrayList<Track>();
                if (mAlbum != null) {
                    tracks = mAlbum.getTracks();
                } else if (mArtist != null) {
                    tracks = mArtist.getTracks();
                } else if (mUserPlaylist != null) {
                    tracks = mUserPlaylist.getTracks();
                } else {
                    tracks.addAll(mTomahawkMainActivity.getCollection().getTracks());
                }
                UserPlaylist playlist = UserPlaylist.fromTrackList("Last used playlist", tracks,
                        (Track) getListAdapter().getItem(position));
                playlist.setCurrentTrackIndex(position);
                ((UserCollection) mTomahawkMainActivity.getCollection())
                        .setCachedPlaylist(playlist);
                Bundle bundle = new Bundle();
                bundle.putBoolean(UserCollection.USERCOLLECTION_PLAYLISTCACHED, true);
                bundle.putLong(PlaybackActivity.PLAYLIST_TRACK_ID,
                        ((Track) getListAdapter().getItem(position)).getId());

                Intent playbackIntent = TomahawkUtils
                        .getIntent(mTomahawkMainActivity, PlaybackActivity.class);
                playbackIntent.putExtra(PlaybackActivity.PLAYLIST_EXTRA, bundle);
                startActivity(playbackIntent);
            }
        }
    }

    /**
     * Called whenever the {@link UserCollection} {@link Loader} has finished
     */
    @Override
    public void onLoadFinished(Loader<Collection> loader, Collection coll) {
        super.onLoadFinished(loader, coll);
        updateAdapter();
    }

    /**
     * Update this {@link TomahawkFragment}'s {@link TomahawkBaseAdapter} content
     */
    public void updateAdapter() {
        List<TomahawkBaseAdapter.TomahawkListItem> items
                = new ArrayList<TomahawkBaseAdapter.TomahawkListItem>();
        TomahawkListAdapter tomahawkListAdapter;
        Collection coll = mTomahawkMainActivity.getCollection();
        if (mAlbum != null) {
            items.addAll(mAlbum.getTracks());
            List<List<TomahawkBaseAdapter.TomahawkListItem>> listArray
                    = new ArrayList<List<TomahawkBaseAdapter.TomahawkListItem>>();
            listArray.add(items);
            if (getListAdapter() == null) {
                tomahawkListAdapter = new TomahawkListAdapter(mTomahawkMainActivity, listArray);
                tomahawkListAdapter.setShowResolvedBy(true);
                tomahawkListAdapter.setShowCategoryHeaders(true);
                tomahawkListAdapter.setShowContentHeader(true, getListView(), mAlbum);
                setListAdapter(tomahawkListAdapter);
            } else {
                ((TomahawkListAdapter) getListAdapter()).setListArray(listArray);
            }
        } else if (mArtist != null) {
            items.addAll(mArtist.getTracks());
            List<List<TomahawkBaseAdapter.TomahawkListItem>> listArray
                    = new ArrayList<List<TomahawkBaseAdapter.TomahawkListItem>>();
            listArray.add(items);
            if (getListAdapter() == null) {
                tomahawkListAdapter = new TomahawkListAdapter(mTomahawkMainActivity, listArray);
                tomahawkListAdapter.setShowResolvedBy(true);
                tomahawkListAdapter.setShowCategoryHeaders(true);
                tomahawkListAdapter.setShowContentHeader(true, getListView(), mArtist);
                setListAdapter(tomahawkListAdapter);
            } else {
                ((TomahawkListAdapter) getListAdapter()).setListArray(listArray);
            }
        } else if (mUserPlaylist != null) {
            mUserPlaylist = coll.getCustomPlaylistById(mUserPlaylist.getId());
            items.addAll(mUserPlaylist.getTracks());
            List<List<TomahawkBaseAdapter.TomahawkListItem>> listArray
                    = new ArrayList<List<TomahawkBaseAdapter.TomahawkListItem>>();
            listArray.add(items);
            if (getListAdapter() == null) {
                tomahawkListAdapter = new TomahawkListAdapter(mTomahawkMainActivity, listArray);
                tomahawkListAdapter.setShowResolvedBy(true);
                tomahawkListAdapter.setShowCategoryHeaders(true);
                tomahawkListAdapter.setShowContentHeader(true, getListView(), mUserPlaylist);
                setListAdapter(tomahawkListAdapter);
            } else {
                ((TomahawkListAdapter) getListAdapter()).setListArray(listArray);
            }
        } else {
            items.addAll(coll.getTracks());
            List<List<TomahawkBaseAdapter.TomahawkListItem>> listArray
                    = new ArrayList<List<TomahawkBaseAdapter.TomahawkListItem>>();
            listArray.add(items);
            if (getListAdapter() == null) {
                tomahawkListAdapter = new TomahawkListAdapter(mTomahawkMainActivity, listArray);
                getListView().setAreHeadersSticky(false);
                setListAdapter(tomahawkListAdapter);
            } else {
                ((TomahawkListAdapter) getListAdapter()).setListArray(listArray);
            }
        }

        getListView().setOnItemClickListener(this);
    }

    /**
     * @return the {@link Album} associated with this {@link TracksFragment}
     */
    public Album getAlbum() {
        return mAlbum;
    }

    /**
     * Resolve the given {@link Album}'s {@link Track}s
     */
    private void resolveAlbum(Album album) {
        if (album != null && album.getTracks() != null) {
            for (Track track : album.getTracks()) {
                if (!track.isResolved()) {
                    String queryId = mPipeline.resolve(track.getName(), track.getAlbum().getName(),
                            track.getArtist().getName());
                    if (queryId != null) {
                        mCorrespondingQueryIds.put(queryId, track);
                        startLoadingAnimation();
                    }
                }
            }
        }
    }
}
